@@ -101,8 +101,11 @@ def student_token_file_runner(host_tmp_dir, student_token_file, instructor_grade
     return pycom, token_location
 
 
-def docker_run_token_file(Dockerfile_location, host_tmp_dir, student_token_file, tag=None, instructor_grade_script=None, fix_user=None):
+
+def docker_run_token_file(Dockerfile_location, host_tmp_dir, student_token_file, tag=None, instructor_grade_script=None, fix_user=None, xvfb=True):
     """
+    xvfb: Control whether to use X-windows. Works on linux. This seems like a good idea when using e.g. gym.
+
     This thingy works:
 
     To build the image, run:
@@ -113,6 +116,10 @@ def docker_run_token_file(Dockerfile_location, host_tmp_dir, student_token_file,
     docker run -v c:/Users/tuhe/Documents/2021/python-docker/tmp:/app python-docker > output.log
 
     """
+    Dockerfile_location = Dockerfile_location.replace("\\", "/")
+    host_tmp_dir = host_tmp_dir.replace("\\", "/")
+    student_token_file = student_token_file.replace("\\", "/")
+
     # A bunch of tests. This is going to be great!
     Dockerfile_location = os.path.abspath(Dockerfile_location)
     assert os.path.exists(Dockerfile_location)
@@ -139,6 +146,7 @@ def docker_run_token_file(Dockerfile_location, host_tmp_dir, student_token_file,
     gscript = instructor_grade_script
 
     student_grade_script = host_tmp_dir + "/" + sources['report_relative_location']
+    student_grade_script = student_grade_script.replace("\\", "/")
     instructor_grade_script = os.path.dirname(student_grade_script) + "/"+os.path.basename(gscript)
     shutil.copy(gscript, instructor_grade_script)
 
@@ -151,6 +159,7 @@ def docker_run_token_file(Dockerfile_location, host_tmp_dir, student_token_file,
         dockname = tag
 
     tmp_grade_file =  sources['name'] + "/" + sources['report_relative_location']
+    tmp_grade_file = tmp_grade_file.replace("\\", "/")
 
     pycom = ".".join( sources['report_module_specification'][:-1] + [os.path.basename(gscript)[:-3],] )
     pycom = "python3 -m " + pycom
@@ -159,6 +168,10 @@ def docker_run_token_file(Dockerfile_location, host_tmp_dir, student_token_file,
         user_cmd = ' --user "$(id -u):$(id -g)" '
     else:
         user_cmd = ''
+
+    if xvfb:
+        user_cmd = " -e DISPLAY=:0 -v /tmp/.X11-unix:/tmp/.X11-unix " + user_cmd
+
     tmp_path = os.path.abspath(host_tmp_dir).replace("\\", "/")
     dcom = f"docker run {user_cmd} -v {tmp_path}:/home {dockname} {pycom}"
     cdcom = f"cd {os.path.dirname(Dockerfile_location)}"
@@ -167,11 +180,14 @@ def docker_run_token_file(Dockerfile_location, host_tmp_dir, student_token_file,
     print(fcom)
     init = time.time() - start
     # thtools.execute_command(fcom.split())
-    subprocess.check_output(fcom, shell=True).decode("utf-8")
+    out = subprocess.check_output(fcom, shell=True).decode("utf-8")
     host_tmp_dir +"/" + os.path.dirname(tmp_grade_file) + "/"
     tokens = glob.glob( os.path.dirname(instructor_grade_script) + "/*.token" )
     for t in tokens:
         print("Source image produced token", t)
     elapsed = time.time() - start
     print("Elapsed time is", elapsed, f"({init=} seconds)")
+    if len(tokens) != 1:
+        print("Wrong number of tokens produced:", len(tokens))
+        print(out)
     return tokens[0]
